@@ -232,7 +232,8 @@ create_lostfound:
 				BTREE_UPDATE_internal_snapshot_node|
 				STR_HASH_must_create) ?:
 		bch2_inode_write_flags(trans, &lostfound_iter, lostfound,
-				       BTREE_UPDATE_internal_snapshot_node);
+				       BTREE_UPDATE_internal_snapshot_node) ?:
+		bch2_trans_commit_lazy(trans, NULL, NULL, BCH_TRANS_COMMIT_no_enospc);
 err:
 	bch_err_msg(c, ret, "creating lost+found");
 	return ret;
@@ -1092,7 +1093,7 @@ int bch2_check_inodes(struct bch_fs *c)
 	CLASS(btree_trans, trans)(c);
 	CLASS(snapshots_seen, s)();
 
-	struct progress_indicator_state progress;
+	struct progress_indicator progress;
 	bch2_progress_init(&progress, c, BIT_ULL(BTREE_ID_inodes));
 
 	return for_each_btree_key_commit(trans, iter, BTREE_ID_inodes,
@@ -1130,9 +1131,7 @@ static int find_oldest_inode_needs_reattach(struct btree_trans *trans,
 			break;
 
 		struct bch_inode_unpacked parent_inode;
-		ret = bch2_inode_unpack(k, &parent_inode);
-		if (ret)
-			break;
+		try(bch2_inode_unpack(k, &parent_inode));
 
 		if (!inode_should_reattach(&parent_inode))
 			break;
@@ -1165,7 +1164,7 @@ static int check_unreachable_inode(struct btree_trans *trans,
 		     "unreachable inode:\n%s",
 		     (bch2_inode_unpacked_to_text(&buf, &inode),
 		      buf.buf)))
-		ret = bch2_reattach_inode(trans, &inode);
+		try(bch2_reattach_inode(trans, &inode));
 fsck_err:
 	return ret;
 }
@@ -1182,7 +1181,7 @@ fsck_err:
  */
 int bch2_check_unreachable_inodes(struct bch_fs *c)
 {
-	struct progress_indicator_state progress;
+	struct progress_indicator progress;
 	bch2_progress_init(&progress, c, BIT_ULL(BTREE_ID_inodes));
 
 	CLASS(btree_trans, trans)(c);
@@ -1708,7 +1707,7 @@ int bch2_check_dirents(struct bch_fs *c)
 	CLASS(snapshots_seen, s)();
 	CLASS(inode_walker, dir)();
 	CLASS(inode_walker, target)();
-	struct progress_indicator_state progress;
+	struct progress_indicator progress;
 	bool need_second_pass = false, did_second_pass = false;
 	int ret;
 again:
@@ -1776,7 +1775,7 @@ int bch2_check_xattrs(struct bch_fs *c)
 	CLASS(btree_trans, trans)(c);
 	CLASS(inode_walker, inode)();
 
-	struct progress_indicator_state progress;
+	struct progress_indicator progress;
 	bch2_progress_init(&progress, c, BIT_ULL(BTREE_ID_xattrs));
 
 	int ret = for_each_btree_key_commit(trans, iter, BTREE_ID_xattrs,
